"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Purpose: 
General Assumptions:

Included Functions/Submodules:
	
"""
##Documentation
## In command line:

##Doesn't work bc webbrowser wont open chrome#### To create documentation html files
##Doesn't work bc webbrowser wont open chrome#### > pyton -m pdoc --html -o SalamanderSpace\Documentation\html SalamanderSpace

## To create local host 
### > pip install pdoc3
### > python -m pdoc --http : SalamanderSpace
## Go to chrome and type in search bar : "localhost:8080"


__all__ = ["FileIO", "Shocks", "Tables", "vlm"]








