"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Package Summary: 
	This package serves as a repository of fundamental relations found throughout Aerospace Engineering
Included Functions/Submodules:
	.Isentropic
	.Shock
"""



__all__ = ["Dimensionless", "Isentropic", "ShockRelations", "Thermodynamic"]